////////////////////////////////////////////////////////////
//Test SL013
//ARM C source code
//LM3S600(6MHz) + EWARM 5.30
////////////////////////////////////////////////////////////   
#include "hw_types.h"
#include "hw_memmap.h"
#include "hw_uart.h"
#include "hw_ints.h"
#include "string.h"
#include "sysctl.h"
#include "gpio.h"
#include "systick.h"
#include "interrupt.h"
#include "uart.h"

//============================================
//	Uart bound define
//============================================
#define  BOUND4800         4800
#define  BOUND9600         9600
#define  BOUND14400        14400
#define  BOUND19200        19200
#define  BOUND28800        28800
#define  BOUND38400        38400
#define  BOUND57600        57600
#define  BOUND115200       115200

//============================================
//	Uart Status define
//============================================
#define UARTSTATUS_FREE    0
#define UARTSTATUS_TX      1
#define UARTSTATUS_TXSUCC  2
#define UARTSTATUS_RXSUCC  3
#define UARTSTATUS_RXERR   4

//============================================
//  Command List, length + command code + data frame
//============================================
const unsigned char __packed Cmd_TrunOnRf[]=    {0x03,0x01,0x01};
const unsigned char __packed Cmd_TrunOffRf[]=   {0x03,0x01,0x00};
const unsigned char __packed Cmd_GetCSN[]=      {0x02,0x10};
const unsigned char __packed Cmd_WriteBlock1[]= {0x1A,0x12,0x00,0x01,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0x11,0x22,0x33,0x44,
                                                 0x55,0x66,0x77,0x88,0x99,0xAA,0xBB,0xCC,0xDD,0xEE,0xFF};
const unsigned char __packed Cmd_ReadBlock1[]=  {0x0A,0x11,0x00,0x01,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
const unsigned char __packed ComIntiPurse2[]=   {0x0E,0x13,0x00,0x02,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x78,0x56,0x34,0x12};
const unsigned char __packed ComReadPurse2[]=   {0x0A,0x14,0x00,0x02,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};                              
const unsigned char __packed ComIncrPurse2[]=   {0x0E,0x15,0x00,0x02,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x02,0x00,0x00,0x00};
const unsigned char __packed ComDecrPurse2[]=   {0x0E,0x16,0x00,0x02,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x01,0x00,0x00,0x00};
const unsigned char __packed pursevalue[]=      {0x79,0x56,0x34,0x12};

//============================================
//  global variable define
//============================================
unsigned char g_cCardType;
unsigned long int g_iTimer;
unsigned char g_bOverTime;
unsigned char g_cUartTxCnt;
unsigned char g_cUartTxDataLen;
unsigned char *g_pUartTxDat;
unsigned char g_cUartTxCheckSum;
unsigned char g_cUartRxCnt;
unsigned char g_cUartRxBuf[60];
unsigned char g_cUartStatus;
unsigned char g_bUartRxHead;
unsigned char g_bUartTxHead;

//============================================
//  procedure define
//============================================
void InitializeSystem(); 
void Start_Time(unsigned long int ms);
void Stop_Time();
void Test_SL013();

void main()
{ InitializeSystem();
  g_iTimer=0;
  while(1) Test_SL013();
}

//============================================
//	system clock interrupt procedure
//============================================
void SysTickIntHandler(void)
{ if(g_iTimer!=0)
  { g_iTimer--;
    if(g_iTimer==0) g_bOverTime=1;
  }
}

//============================================
//  initialize system hardware config
//  system clock,uart0
//============================================
void InitializeSystem()
{	
  //init sysclock
  //period=1ms
  SysCtlClockSet(SYSCTL_SYSDIV_4|SYSCTL_USE_PLL|SYSCTL_OSC_MAIN|SYSCTL_XTAL_6MHZ);
  SysTickPeriodSet(SysCtlClockGet()/1000);
  SysTickEnable();
  SysTickIntEnable();
  
  //init Uart
  //baud=19200
  SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
  SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
  GPIOPinTypeUART(GPIO_PORTA_BASE,GPIO_PIN_0|GPIO_PIN_1);
  UARTDisable(UART0_BASE);
  UARTIntClear(UART0_BASE,UART_INT_OE|UART_INT_BE|UART_INT_PE|UART_INT_FE|UART_INT_RT|UART_INT_TX|UART_INT_RX);
  UARTIntDisable(UART0_BASE,UART_INT_OE|UART_INT_BE|UART_INT_PE|UART_INT_FE|UART_INT_RT|UART_INT_TX|UART_INT_RX);
  UARTConfigSetExpClk(UART0_BASE,SysCtlClockGet(),BOUND19200,(UART_CONFIG_WLEN_8|UART_CONFIG_STOP_ONE|UART_CONFIG_PAR_NONE));
  UARTFIFOLevelSet(UART0_BASE,UART_FIFO_TX1_8,UART_FIFO_RX1_8);
  HWREG(UART0_BASE+UART_O_LCRH) &= ~(UART_LCRH_FEN);
  UARTIntEnable(UART0_BASE,UART_INT_RX|UART_INT_TX);
  IntEnable(INT_UART0);
  g_cUartStatus=UARTSTATUS_FREE;
  g_cUartRxCnt=0;
  
  //interrupt enable
  IntMasterEnable();
}

//============================================
//  start timer
//============================================
void Start_Time(unsigned long int ms)
{ g_iTimer=ms;
  g_bOverTime=0;
}

//============================================
//  stop timer
//============================================
void Stop_Time()
{ g_iTimer=0;
}

void UARTISR_TX013()
{ unsigned char tmp;
  if(g_cUartTxCnt==g_cUartTxDataLen)
  { if(g_cUartStatus==UARTSTATUS_TX)
    { g_cUartStatus=UARTSTATUS_TXSUCC;
      g_cUartRxCnt=0;
      g_bUartRxHead=0;
      UARTCharPut(UART0_BASE,g_cUartTxCheckSum);
      while(UARTBusy(UART0_BASE));
    }
  }
  else if(g_cUartTxCnt<g_cUartTxDataLen)
  { if(g_bUartTxHead)
    { g_bUartTxHead=0;
      UARTCharPut(UART0_BASE,0);
      while(UARTBusy(UART0_BASE));
    }
    else
    { tmp=g_pUartTxDat[g_cUartTxCnt];
      g_cUartTxCheckSum=g_cUartTxCheckSum^tmp;
      g_cUartTxCnt++;
      if(tmp==0xaa) g_bUartTxHead=1;else g_bUartTxHead=0;
      UARTCharPut(UART0_BASE,tmp);
      while(UARTBusy(UART0_BASE));
    }
  }
}

void UARTISR_RX013(unsigned char dat)
{ unsigned char CheckSum;
  unsigned char counter;
  if(g_bUartRxHead)
  { if(dat!=0) g_cUartRxCnt=0;
    g_bUartRxHead=0;
  }
  else
  { g_cUartRxBuf[g_cUartRxCnt]=dat;
    g_cUartRxCnt++;
    if(dat==0xaa) g_bUartRxHead=1;
    if(g_cUartRxCnt==g_cUartRxBuf[0]+1)
    { CheckSum=0;
      for(counter=0;counter<g_cUartRxCnt;counter++) CheckSum=CheckSum^g_cUartRxBuf[counter];
      if(CheckSum==0) g_cUartStatus=UARTSTATUS_RXSUCC;
      else g_cUartStatus=UARTSTATUS_RXERR;
    }
  }
}

void UART0_ISR(void)
{ unsigned long flag_int;
  unsigned char tmp;
  flag_int=UARTIntStatus(UART0_BASE,true);
  UARTIntClear(UART0_BASE,flag_int);
  if((flag_int&UART_INT_TX)==UART_INT_TX) UARTISR_TX013();
  if((flag_int&UART_INT_RX)==UART_INT_RX) 
  { tmp=UARTCharGetNonBlocking(UART0_BASE);
    UARTISR_RX013(tmp);
  }
}

//============================================
//  send buffer by uart0
//============================================
void SendBuf_UART(unsigned char *dat,unsigned char len)
{ g_cUartStatus=UARTSTATUS_TX;
  g_pUartTxDat=dat;
  g_cUartTxDataLen=len;
  g_cUartTxCnt=0;
  g_cUartTxCheckSum=0;
  g_bUartTxHead=0;
  UARTIntDisable(UART0_BASE,UART_INT_TX);
  UARTCharPut(UART0_BASE,0xaa);
  while(UARTBusy(UART0_BASE));
  UARTCharPut(UART0_BASE,0xbb);
  while(UARTBusy(UART0_BASE));
  UARTIntEnable(UART0_BASE,UART_INT_TX);
  UARTISR_TX013();
}

//============================================
//  SL013 test procedure
//============================================
void Test_SL013()
{  
  #define RCVCMD_SL013 g_cUartRxBuf[1]
  #define RCVSTA_SL013 g_cUartRxBuf[2]
  g_cCardType=0xff;
  SendBuf_UART((unsigned char *)(Cmd_GetCSN),sizeof(Cmd_GetCSN));
  Start_Time(300);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  Stop_Time();
  if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL013!=0x10)||(RCVSTA_SL013!=0)) return;
  g_cCardType=g_cUartRxBuf[7];
  if((g_cCardType==0)||(g_cCardType==1))
  { 
    //Write block1 data: 0x00112233445566778899AABBCCDDEEFF 
    SendBuf_UART((unsigned char*)(Cmd_WriteBlock1),sizeof(Cmd_WriteBlock1));
    Start_Time(300);
    while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
    Stop_Time();
    if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL013!=0x12)||(RCVSTA_SL013!=0)) return;
    
    //Read data from block1 
    SendBuf_UART((unsigned char*)(Cmd_ReadBlock1),sizeof(Cmd_ReadBlock1));
    Start_Time(300);
    while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
    Stop_Time();
    if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL013!=0x11)||(RCVSTA_SL013!=0)) return;
    
    //Compare data 
    if(memcmp(&g_cUartRxBuf[3],(unsigned char *)(&Cmd_WriteBlock1[10]),16)!=0) return;
    
    //Initialize block2 to one pursevalue = 0x12345678 
    SendBuf_UART((unsigned char*)(ComIntiPurse2),sizeof(ComIntiPurse2));
    Start_Time(300);
    while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
    Stop_Time();
    if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL013!=0x13)||(RCVSTA_SL013!=0)) return;
    
    //Incrment 2 to purse
    SendBuf_UART((unsigned char*)(ComIncrPurse2),sizeof(ComIncrPurse2));
    Start_Time(300);
    while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
    Stop_Time();
    if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL013!=0x15)||(RCVSTA_SL013!=0)) return;
    
    //Decrment 1 to purse 
    SendBuf_UART((unsigned char*)(ComDecrPurse2),sizeof(ComDecrPurse2));
    Start_Time(300);
    while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
    Stop_Time();
    if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL013!=0x16)||(RCVSTA_SL013!=0)) return;
    
    //Read purse value, should be equal to 0x12345679  
    SendBuf_UART((unsigned char*)(ComReadPurse2),sizeof(ComReadPurse2));
    Start_Time(300);
    while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
    Stop_Time();
    if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL013!=0x14)||(RCVSTA_SL013!=0)) return;
    if(memcmp(&pursevalue[0],&g_cUartRxBuf[3],4)!=0) return;
    //test ok		
  }
}

